@@ -1,179 +1,157 @@
-# -*- coding: iso8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003, 2004 Edgewall Software
-# Copyright (C) 2003, 2004 Daniel Lundin <daniel@edgewall.com>
+# Copyright (C) 2003-2005 Edgewall Software
+# Copyright (C) 2003-2005 Daniel Lundin <daniel@edgewall.com>
+# All rights reserved.
 #
-# Trac is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; either version 2 of the
-# License, or (at your option) any later version.
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
 #
-# Trac is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://projects.edgewall.com/trac/.
 #
 # Author: Daniel Lundin <daniel@edgewall.com>
 
+from trac.config import default_dir
+from trac.db import Table, Column, Index
 
 # Database version identifier. Used for automatic upgrades.
-db_version = 7
+db_version = 16
 
-def __mkreports(reps):
+def __mkreports(reports):
     """Utility function used to create report data in same syntax as the
     default data. This extra step is done to simplify editing the default
     reports."""
     result = []
-    i = 1
-    for r in reps:
-        result.append ((i, None, r[0], r[2], r[1]))
-        i = i + 1
+    for report in reports:
+        result.append((None, report[0], report[2], report[1]))
     return result
 
 
 ##
-## Default data
+## Database schema
 ##
 
-schema = """
-CREATE TABLE revision (
-        rev             integer PRIMARY KEY,
-        time            integer,
-        author          text,
-        message         text
-);
-CREATE TABLE node_change (
-        rev             integer,
-        name            text,
-        change          char(1),
-        UNIQUE(rev, name, change)
-);
-CREATE TABLE auth_cookie (
-        cookie          text,
-        name            text,
-        ipnr            text,
-        time            integer,
-        UNIQUE(cookie, name, ipnr)
-);
-CREATE TABLE enum (
-        type            text,
-        name            text,
-        value           text,
-        UNIQUE(name,type)
-);
-CREATE TABLE system (
-        name            text PRIMARY KEY,
-        value           text,
-        UNIQUE(name)
-);
-CREATE TABLE lock (
-        name            text PRIMARY KEY,
-        owner           text,
-        ipnr            text,
-        time            integer,
-        UNIQUE(name)
-);
-CREATE TABLE ticket (
-        id              integer PRIMARY KEY,
-        time            integer,        -- the time it was created
-        changetime      integer,
-        component       text,
-        severity        text,
-        priority        text,
-        owner           text,           -- who is this ticket assigned to
-        reporter        text,
-        cc              text,           -- email addresses to notify
-        url             text,           -- url related to this ticket
-        version         text,           -- 
-        milestone       text,           -- 
-        status          text,
-        resolution      text,
-        summary         text,           -- one-line summary
-        description     text,           -- problem description (long)
-        keywords        text
-);
-CREATE TABLE ticket_change (
-        ticket          integer,
-        time            integer,
-        author          text,
-        field           text,
-        oldvalue        text,
-        newvalue        text,
-        UNIQUE(ticket, time, field)
-);
-CREATE TABLE ticket_custom (
-       ticket               integer,
-       name             text,
-       value            text,
-       UNIQUE(ticket,name)
-);
-CREATE TABLE report (
-        id              integer PRIMARY KEY,
-        author          text,
-        title           text,
-        sql             text,
-        description     text
-);
-CREATE TABLE permission (
-        username        text,           -- 
-        action          text,           -- allowable activity
-        UNIQUE(username,action)
-);
-CREATE TABLE component (
-         name            text PRIMARY KEY,
-         owner           text
-);
-CREATE TABLE milestone (
-         id              integer PRIMARY KEY,
-         name            text,
-         time            integer,
-         descr           text,
-         UNIQUE(name)
-);
-CREATE TABLE version (
-         name            text PRIMARY KEY,
-         time            integer
-);
-CREATE TABLE wiki (
-         name            text,
-         version         integer,
-         time            integer,
-         author          text,
-         ipnr            text,
-         text            text,
-         comment         text,
-         readonly        integer,
-         UNIQUE(name,version)
-);
-CREATE TABLE attachment (
-         type            text,
-         id              text,
-         filename        text,
-         size            integer,
-         time            integer,
-         description     text,
-         author          text,
-         ipnr            text,
-         UNIQUE(type,id,filename)
-);
+schema = [
+    # Common
+    Table('system', key='name')[
+        Column('name'),
+        Column('value')],
+    Table('permission', key=('username', 'action'))[
+        Column('username'),
+        Column('action')],
+    Table('auth_cookie', key=('cookie', 'ipnr', 'name'))[
+        Column('cookie'),
+        Column('name'),
+        Column('ipnr'),
+        Column('time', type='int')],
+    Table('session', key=('sid', 'authenticated', 'var_name'))[
+        Column('sid'),
+        Column('authenticated', type='int'),
+        Column('var_name'),
+        Column('var_value')],
 
-CREATE TABLE session (
-         sid             text,
-         username        text,
-         var_name        text,
-         var_value       text,
-         UNIQUE(sid,var_name)
-);
+    # Attachments
+    Table('attachment', key=('type', 'id', 'filename'))[
+        Column('type'),
+        Column('id'),
+        Column('filename'),
+        Column('size', type='int'),
+        Column('time', type='int'),
+        Column('description'),
+        Column('author'),
+        Column('ipnr')],
+
+    # Wiki system
+    Table('wiki', key=('name', 'version'))[
+        Column('name'),
+        Column('version', type='int'),
+        Column('time', type='int'),
+        Column('author'),
+        Column('ipnr'),
+        Column('text'),
+        Column('comment'),
+        Column('readonly', type='int'),
+        Index(['time'])],
+
+    # Version control cache
+    Table('revision', key='rev')[
+        Column('rev'),
+        Column('time', type='int'),
+        Column('author'),
+        Column('message'),
+        Index(['time'])],
+    Table('node_change', key=('rev', 'path', 'change'))[
+        Column('rev'),
+        Column('path'),
+        Column('kind', size=1),
+        Column('change', size=1),
+        Column('base_path'),
+        Column('base_rev'),
+        Index(['rev'])],
+
+    # Ticket system
+    Table('ticket', key='id')[
+        Column('id', auto_increment=True),
+        Column('type'),
+        Column('time', type='int'),
+        Column('changetime', type='int'),
+        Column('component'),
+        Column('severity'),
+        Column('priority'),
+        Column('owner'),
+        Column('reporter'),
+        Column('cc'),
+        Column('version'),
+        Column('milestone'),
+        Column('status'),
+        Column('resolution'),
+        Column('summary'),
+        Column('description'),
+        Column('keywords'),
+        Index(['time']),
+        Index(['status'])],    
+    Table('ticket_change', key=('ticket', 'time', 'field'))[
+        Column('ticket', type='int'),
+        Column('time', type='int'),
+        Column('author'),
+        Column('field'),
+        Column('oldvalue'),
+        Column('newvalue'),
+        Index(['ticket', 'time'])],
+    Table('ticket_custom', key=('ticket', 'name'))[
+        Column('ticket', type='int'),
+        Column('name'),
+        Column('value')],
+    Table('enum', key=('type', 'name'))[
+        Column('type'),
+        Column('name'),
+        Column('value')],
+    Table('component', key='name')[
+        Column('name'),
+        Column('owner'),
+        Column('description')],
+    Table('milestone', key='name')[
+        Column('name'),
+        Column('due', type='int'),
+        Column('completed', type='int'),
+        Column('description')],
+    Table('version', key='name')[
+        Column('name'),
+        Column('time', type='int'),
+        Column('description')],
+
+    # Report system
+    Table('report', key='id')[
+        Column('id', auto_increment=True),
+        Column('author'),
+        Column('title'),
+        Column('sql'),
+        Column('description')],
+]
 
-CREATE INDEX node_change_idx ON node_change(rev);
-CREATE INDEX ticket_change_idx  ON ticket_change(ticket, time);
-CREATE INDEX wiki_idx           ON wiki(name,version);
-CREATE INDEX session_idx        ON session(sid,var_name);
-"""
 
 ##
 ## Default Reports
@@ -188,7 +166,7 @@ reports = (
 """,
 """
 SELECT p.value AS __color__,
-   id AS ticket, summary, component, version, milestone, severity, 
+   id AS ticket, summary, component, version, milestone, t.type AS type, 
    (CASE status WHEN 'assigned' THEN owner||' *' ELSE owner END) AS owner,
    time AS created,
    changetime AS _changetime, description AS _description,
@@ -196,7 +174,7 @@ SELECT p.value AS __color__,
   FROM ticket t, enum p
   WHERE status IN ('new', 'assigned', 'reopened') 
 AND p.name = t.priority AND p.type = 'priority'
-  ORDER BY p.value, milestone, severity, time
+  ORDER BY p.value, milestone, t.type, time
 """),
 #----------------------------------------------------------------------------
  ('Active Tickets by Version',
@@ -210,7 +188,7 @@ for useful RSS export.
 """
 SELECT p.value AS __color__,
    version AS __group__,
-   id AS ticket, summary, component, version, severity, 
+   id AS ticket, summary, component, version, t.type AS type, 
    (CASE status WHEN 'assigned' THEN owner||' *' ELSE owner END) AS owner,
    time AS created,
    changetime AS _changetime, description AS _description,
@@ -218,7 +196,7 @@ SELECT p.value AS __color__,
   FROM ticket t, enum p
   WHERE status IN ('new', 'assigned', 'reopened') 
 AND p.name = t.priority AND p.type = 'priority'
-  ORDER BY (version IS NULL),version, p.value, severity, time
+  ORDER BY (version IS NULL),version, p.value, t.type, time
 """),
 #----------------------------------------------------------------------------
 ('All Tickets by Milestone',
@@ -232,7 +210,7 @@ for useful RSS export.
 """
 SELECT p.value AS __color__,
    milestone||' Release' AS __group__,
-   id AS ticket, summary, component, version, severity, 
+   id AS ticket, summary, component, version, t.type AS type, 
    (CASE status WHEN 'assigned' THEN owner||' *' ELSE owner END) AS owner,
    time AS created,
    changetime AS _changetime, description AS _description,
@@ -240,7 +218,7 @@ SELECT p.value AS __color__,
   FROM ticket t, enum p
   WHERE status IN ('new', 'assigned', 'reopened') 
 AND p.name = t.priority AND p.type = 'priority'
-  ORDER BY (milestone IS NULL),milestone, p.value, severity, time
+  ORDER BY (milestone IS NULL),milestone, p.value, t.type, time
 """),
 #----------------------------------------------------------------------------
 ('Assigned, Active Tickets by Owner',
@@ -251,13 +229,13 @@ List assigned tickets, group by ticket owner, sorted by priority.
 
 SELECT p.value AS __color__,
    owner AS __group__,
-   id AS ticket, summary, component, milestone, severity, time AS created,
+   id AS ticket, summary, component, milestone, t.type AS type, time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
   FROM ticket t,enum p
   WHERE status = 'assigned'
 AND p.name=t.priority AND p.type='priority'
-  ORDER BY owner, p.value, severity, time
+  ORDER BY owner, p.value, t.type, time
 """),
 #----------------------------------------------------------------------------
 ('Assigned, Active Tickets by Owner (Full Description)',
@@ -268,13 +246,13 @@ This report demonstrates the use of full-row display.
 """
 SELECT p.value AS __color__,
    owner AS __group__,
-   id AS ticket, summary, component, milestone, severity, time AS created,
+   id AS ticket, summary, component, milestone, t.type AS type, time AS created,
    description AS _description_,
    changetime AS _changetime, reporter AS _reporter
   FROM ticket t, enum p
   WHERE status = 'assigned'
 AND p.name = t.priority AND p.type = 'priority'
-  ORDER BY owner, p.value, severity, time
+  ORDER BY owner, p.value, t.type, time
 """),
 #----------------------------------------------------------------------------
 ('All Tickets By Milestone  (Including closed)',
@@ -290,13 +268,13 @@ SELECT p.value AS __color__,
         (CASE owner WHEN '$USER' THEN 'font-weight: bold' END)
     END) AS __style__,
    id AS ticket, summary, component, status, 
-   resolution,version, severity, priority, owner,
+   resolution,version, t.type AS type, priority, owner,
    changetime AS modified,
    time AS _time,reporter AS _reporter
   FROM ticket t,enum p
   WHERE p.name=t.priority AND p.type='priority'
   ORDER BY (milestone IS NULL), milestone DESC, (status = 'closed'), 
-        (CASE status WHEN 'closed' THEN modified ELSE -p.value END) DESC
+        (CASE status WHEN 'closed' THEN modified ELSE (-1)*p.value END) DESC
 """),
 #----------------------------------------------------------------------------
 ('My Tickets',
@@ -309,13 +287,13 @@ logged in user when executed.
 SELECT p.value AS __color__,
    (CASE status WHEN 'assigned' THEN 'Assigned' ELSE 'Owned' END) AS __group__,
    id AS ticket, summary, component, version, milestone,
-   severity, priority, time AS created,
+   t.type AS type, priority, time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
   FROM ticket t, enum p
   WHERE t.status IN ('new', 'assigned', 'reopened') 
 AND p.name = t.priority AND p.type = 'priority' AND owner = '$USER'
-  ORDER BY (status = 'assigned') DESC, p.value, milestone, severity, time
+  ORDER BY (status = 'assigned') DESC, p.value, milestone, t.type, time
 """),
 #----------------------------------------------------------------------------
 ('Active Tickets, Mine first',
@@ -329,7 +307,7 @@ SELECT p.value AS __color__,
      WHEN '$USER' THEN 'My Tickets' 
      ELSE 'Active Tickets' 
     END) AS __group__,
-   id AS ticket, summary, component, version, milestone, severity, 
+   id AS ticket, summary, component, version, milestone, t.type AS type, 
    (CASE status WHEN 'assigned' THEN owner||' *' ELSE owner END) AS owner,
    time AS created,
    changetime AS _changetime, description AS _description,
@@ -337,7 +315,7 @@ SELECT p.value AS __color__,
   FROM ticket t, enum p
   WHERE status IN ('new', 'assigned', 'reopened') 
 AND p.name = t.priority AND p.type = 'priority'
-  ORDER BY (owner = '$USER') DESC, p.value, milestone, severity, time
+  ORDER BY (owner = '$USER') DESC, p.value, milestone, t.type, time
 """))
 
 
@@ -351,16 +329,14 @@ data = (('component',
                (('component1', 'somebody'),
                 ('component2', 'somebody'))),
            ('milestone',
-             ('name', 'time'),
-               (('', 0), 
-                ('milestone1', 0),
-                ('milestone2', 0),
-                ('milestone3', 0),
-                ('milestone4', 0))),
+             ('name', 'due', 'completed'),
+               (('milestone1', 0, 0),
+                ('milestone2', 0, 0),
+                ('milestone3', 0, 0),
+                ('milestone4', 0, 0))),
            ('version',
              ('name', 'time'),
-               (('', 0),
-                ('1.0', 0),
+               (('1.0', 0),
                 ('2.0', 0))),
            ('enum',
              ('type', 'name', 'value'),
@@ -373,18 +349,14 @@ data = (('component',
                 ('resolution', 'wontfix', 3),
                 ('resolution', 'duplicate', 4),
                 ('resolution', 'worksforme', 5),
-                ('severity', 'blocker', 1),
-                ('severity', 'critical', 2),
-                ('severity', 'major', 3),
-                ('severity', 'normal', 4),
-                ('severity', 'minor', 5),
-                ('severity', 'trivial', 6),
-                ('severity', 'enhancement', 7),
-                ('priority', 'highest', 1),
-                ('priority', 'high', 2),
-                ('priority', 'normal', 3),
-                ('priority', 'low', 4),
-                ('priority', 'lowest', 5))),
+                ('priority', 'blocker', 1),
+                ('priority', 'critical', 2),
+                ('priority', 'major', 3),
+                ('priority', 'minor', 4),
+                ('priority', 'trivial', 5),
+                ('ticket_type', 'defect', 1),
+                ('ticket_type', 'enhancement', 2),
+                ('ticket_type', 'task', 3))),
            ('permission',
              ('username', 'action'),
                (('anonymous', 'LOG_VIEW'),
@@ -407,43 +379,71 @@ data = (('component',
              ('name', 'value'),
                (('database_version', str(db_version)),)),
            ('report',
-             ('id', 'author', 'title', 'sql', 'description'),
+             ('author', 'title', 'sql', 'description'),
                __mkreports(reports)))
 
 default_config = \
- (('trac', 'htdocs_location', '/trac/'),
-  ('trac', 'repository_dir', '/var/svn/myrep'),
-  ('trac', 'templates_dir', '/usr/lib/trac/templates'),
+ (('trac', 'repository_dir', ''),
+  ('trac', 'templates_dir', default_dir('templates')),
   ('trac', 'database', 'sqlite:db/trac.db'),
   ('trac', 'default_charset', 'iso-8859-15'),
+  ('trac', 'default_handler', 'WikiModule'),
+  ('trac', 'check_auth_ip', 'true'),
+  ('trac', 'ignore_auth_case', 'false'),
+  ('trac', 'metanav', 'login,logout,settings,help,about'),
+  ('trac', 'mainnav', 'wiki,timeline,roadmap,browser,tickets,newticket,search'),
+  ('trac', 'permission_store', 'DefaultPermissionStore'),
   ('logging', 'log_type', 'none'),
   ('logging', 'log_file', 'trac.log'),
   ('logging', 'log_level', 'DEBUG'),
   ('project', 'name', 'My Project'),
   ('project', 'descr', 'My example project'),
   ('project', 'url', 'http://example.com/'),
-  ('project', 'icon', 'trac.ico'),
+  ('project', 'icon', 'common/trac.ico'),
   ('project', 'footer',
    ' Visit the Trac open source project at<br />'
    '<a href="http://trac.edgewall.com/">http://trac.edgewall.com/</a>'),
   ('ticket', 'default_version', ''),
-  ('ticket', 'default_severity', 'normal'),
-  ('ticket', 'default_priority', 'normal'),
+  ('ticket', 'default_type', 'defect'),
+  ('ticket', 'default_priority', 'major'),
   ('ticket', 'default_milestone', ''),
   ('ticket', 'default_component', 'component1'),
+  ('ticket', 'restrict_owner', 'false'),
   ('header_logo', 'link', 'http://trac.edgewall.com/'),
-  ('header_logo', 'src', 'trac_banner.png'),
+  ('header_logo', 'src', 'common/trac_banner.png'),
   ('header_logo', 'alt', 'Trac'),
   ('header_logo', 'width', '236'),
   ('header_logo', 'height', '73'),
   ('attachment', 'max_size', '262144'),
-  ('diff', 'tab_width', '8'),
   ('mimeviewer', 'enscript_path', 'enscript'),
+  ('mimeviewer', 'php_path', 'php'),
+  ('mimeviewer', 'tab_width', '8'),
+  ('mimeviewer', 'max_preview_size', '262144'),
   ('notification', 'smtp_enabled', 'false'),
   ('notification', 'smtp_server', 'localhost'),
+  ('notification', 'smtp_port', '25'),
+  ('notification', 'smtp_user', ''),
+  ('notification', 'smtp_password', ''),
   ('notification', 'smtp_always_cc', ''),
+  ('notification', 'always_notify_owner', 'false'),
   ('notification', 'always_notify_reporter', 'false'),
   ('notification', 'smtp_from', 'trac@localhost'),
   ('notification', 'smtp_replyto', 'trac@localhost'),
-  ('timeline', 'changeset_show_files', '0'))
-   
+  ('timeline', 'default_daysback', '30'),
+  ('timeline', 'changeset_show_files', '0'),
+  ('timeline', 'ticket_show_details', 'false'),
+  ('browser', 'hide_properties', 'svk:merge'),
+  ('wiki', 'ignore_missing_pages', 'false'),
+)
+
+default_components = ('trac.About', 'trac.attachment', 
+                      'trac.mimeview.enscript', 'trac.mimeview.patch',
+                      'trac.mimeview.php', 'trac.mimeview.rst',
+                      'trac.mimeview.silvercity', 'trac.mimeview.txtl',
+                      'trac.Search', 'trac.Settings',
+                      'trac.ticket.query', 'trac.ticket.report',
+                      'trac.ticket.roadmap', 'trac.ticket.web_ui',
+                      'trac.Timeline',
+                      'trac.versioncontrol.web_ui',
+                      'trac.wiki.macros', 'trac.wiki.web_ui',
+                      'trac.web.auth')
